@@ -6,6 +6,10 @@
 #'
 #' @slot pdf     cf. parameter 'pdf'
 #' @slot support cf. parameter 'support'
+#' @slot pivots normalized pivots for integration rule (in [-1, 1])
+#'     the actual pivots are scaled to the support of the prior
+#' @slot weights weights of of integration rule at \code{pivots} for
+#'     approximating integrals over \code{delta}
 #'
 #' @seealso Discrete priors are supported via \code{\link{PointMassPrior}}
 #'
@@ -13,7 +17,9 @@
 #' @exportClass ContinuousPrior
 setClass("ContinuousPrior", representation(
         pdf     = "function",
-        support = "numeric"
+        support = "numeric",
+        pivots  = "numeric",
+        weights = "numeric"
         ),
     contains = "Prior")
 
@@ -21,6 +27,7 @@ setClass("ContinuousPrior", representation(
 #' @param pdf                 vectorized univariate PDF function
 #' @param support             numeric vector of length two with the bounds of
 #'     the compact interval on which the pdf is positive.
+#' @template order
 #' @template tighten_support
 #' @template check_normalization
 #' @template label
@@ -32,6 +39,7 @@ setClass("ContinuousPrior", representation(
 #' @export
 ContinuousPrior <- function(pdf,
                             support,
+                            order = 10,
                             label = NA_character_,
                             tighten_support = FALSE,
                             check_normalization = TRUE) {
@@ -62,7 +70,13 @@ ContinuousPrior <- function(pdf,
         pdf     <- function(theta) pdf_old(theta) / norm
     }
 
-    new("ContinuousPrior", pdf = pdf, support = support, label = label)
+    rule   <- GaussLegendreRule(order)
+    h      <- (support[2] - support[1]) / 2
+    pivots <- h * rule$nodes + (h + support[1])
+
+
+    new("ContinuousPrior", pdf = pdf, support = support,
+        pivots = pivots, weights = rule$weights, label = label)
 }
 
 
@@ -83,17 +97,11 @@ setMethod("bounds", signature("ContinuousPrior"),
 #' )
 #' # > 0.3
 #'
-#' @param rel.tol \code{numeric}, relative tolerance used in adaptive gaussian
-#'     quadrature to evaluate the integral
-#'
 #' @rdname expectation
 #' @export
 setMethod("expectation", signature("ContinuousPrior", "function"),
-    function(dist, f, rel.tol = .001, ...) {
-        stats::integrate(
-            function(theta) f(theta) * dist@pdf(theta),
-            dist@support[1], dist@support[2], rel.tol = rel.tol
-        )$value
+    function(dist, f, ...) {
+        gauss_quad(f(dist@pivots) * dist@pdf(dist@pivots), dist@support[1], dist@support[2], dist@weights)
     })
 
 
@@ -108,29 +116,21 @@ setMethod("expectation", signature("ContinuousPrior", "function"),
 #' @export
 setMethod("condition", signature("ContinuousPrior", "numeric"),
     function(dist, interval, ...) {
-        if (length(interval) != 2)
-            stop("interval must be of length 2")
-        if (any(!is.finite(interval)))
-            stop("interval must be finite")
-        if (diff(interval) < 0)
-            stop("interval[2] must be larger or equal to interval[1]")
-
+        if (length(interval) != 2) stop("interval must be of length 2")
+        if (any(!is.finite(interval))) stop("interval must be finite")
+        if (diff(interval) < 0) stop("interval[2] must be larger or equal to interval[1]")
         interval[1] <- max(interval[1], dist@support[1])
         interval[2] <- min(interval[2], dist@support[2])
-        if (diff(interval) < 0)
-            stop("resulting interval is empty")
-        # compute new normalizing constant
-        z <- stats::integrate(dist@pdf, interval[1], interval[2], abs.tol = .00001)$value
-        new_pdf <- function(theta) {
-            ifelse(interval[1] <= theta & theta <= interval[2],
-                dist@pdf(theta) / z,
-                0
-            )
-        }
-        ContinuousPrior(
-            new_pdf,
-            interval
-        )
+        if (diff(interval) < 0) stop("resulting interval is empty")
+        z       <- stats::integrate(dist@pdf, interval[1], interval[2], abs.tol = .001)$value
+        new_pdf <- function(theta) ifelse(interval[1] <= theta & theta <= interval[2], dist@pdf(theta)/z, 0)
+        h_old   <- (dist@support[2] - dist@support[1]) / 2
+        nodes   <- (dist@pivots - (h_old + dist@support[1])) / h_old
+        h_new   <- (interval[2] - interval[1]) / 2
+        pivots  <- h_new * nodes + (h_new + interval[1])
+
+        new("ContinuousPrior", pdf = new_pdf, support = interval,
+            pivots = pivots, weights = dist@weights, label = dist@label)
     })
 
 
@@ -143,15 +143,12 @@ setMethod("condition", signature("ContinuousPrior", "numeric"),
 #' @rdname predictive_pdf
 #' @export
 setMethod("predictive_pdf", signature("DataDistribution", "ContinuousPrior", "numeric"),
-    function(dist, prior, x1, n1, k = 33, ...) {
+    function(dist, prior, x1, n1, k = 10*(prior@support[2] - prior@support[1]) + 1, ...) {
         piv  <- seq(prior@support[1], prior@support[2], length.out = k)
         mass <- sapply(piv, prior@pdf)
         mass <- mass / sum(mass) # (renormalize!)
-        res  <- numeric(length(x1))
-        for (i in 1:k) {
-            res <- res + mass[i] * probability_density_function(dist, x1, n1, piv[i])
-        }
-        return(res)
+        grid <- expand.grid(x1 = x1, piv = piv)
+        (matrix(probability_density_function(dist, grid$x1, n1, grid$piv), nrow = length(x1)) %*% mass)[, 1]
     })
 
 
@@ -164,15 +161,12 @@ setMethod("predictive_pdf", signature("DataDistribution", "ContinuousPrior", "nu
 #' @rdname predictive_cdf
 #' @export
 setMethod("predictive_cdf", signature("DataDistribution", "ContinuousPrior", "numeric"),
-    function(dist, prior, x1, n1, k = 33, ...) {
+    function(dist, prior, x1, n1, k = 10*(prior@support[2] - prior@support[1]) + 1, ...) {
         piv  <- seq(prior@support[1], prior@support[2], length.out = k)
         mass <- sapply(piv, prior@pdf)
         mass <- mass / sum(mass) # (renormalize!)
-        res  <- numeric(length(x1))
-        for (i in 1:k) {
-            res <- res + mass[i] * cumulative_distribution_function(dist, x1, n1, piv[i])
-        }
-        return(res)
+        grid <- expand.grid(x1 = x1, piv = piv)
+        (matrix(cumulative_distribution_function(dist, grid$x1, n1, grid$piv), nrow = length(x1)) %*% mass)[, 1]
     })
 
 
@@ -180,26 +174,19 @@ setMethod("predictive_cdf", signature("DataDistribution", "ContinuousPrior", "nu
 #' tmp <- ContinuousPrior(function(x) dunif(x, .2, .4), c(.2, .4))
 #' posterior(Normal(), tmp, 2, 20)
 #'
-#' @template tighten_support
-#' @template check_normalization
-#'
 #' @rdname posterior
 #' @export
 setMethod("posterior", signature("DataDistribution", "ContinuousPrior", "numeric"),
-    function(dist, prior, x1, n1, tighten_support = FALSE, check_normalization = FALSE, ...) {
-        if (length(x1) != 1)
-            stop("no vectorized version in x1")
-        prop_pdf <- function(theta) {
-            probability_density_function(dist, x1, n1, theta) * prior@pdf(theta)
-        }
-        z <- stats::integrate(
-            prop_pdf, prior@support[1], prior@support[2], abs.tol = .00001
-        )$value
-        ContinuousPrior(
-            function(theta) prop_pdf(theta) / z,
-            prior@support,
-            tighten_support     = tighten_support,
-            check_normalization = check_normalization
+    function(dist, prior, x1, n1, ...) {
+        if (length(x1) != 1) stop("no vectorized version in x1")
+        prop_pdf <- function(theta) probability_density_function(dist, x1, n1, theta) * prior@pdf(theta)
+        z        <- gauss_quad(prop_pdf(prior@pivots), prior@support[1], prior@support[2], prior@weights)
+        new("ContinuousPrior",
+            pdf     = function(theta) prop_pdf(theta) / z,
+            support = prior@support,
+            pivots  = prior@pivots,
+            weights = prior@weights,
+            label   = prior@label
         )
     })
 
