@@ -9,7 +9,6 @@
 #' @template design
 #' @template optimization
 #' @template dotdotdot
-#' @template object
 #' @param e1 left hand side (score or numeric)
 #' @param e2 right hand side (score or numeric)
 #'
@@ -36,7 +35,8 @@ NULL
 
 
 
-setClass("Constraint")
+setClass("Constraint", representation(label = "character"),
+         prototype(label = NA_character_))
 setClass("ConditionalConstraint", representation(
         score = "ConditionalScore",
         rhs   = "numeric"
@@ -59,10 +59,17 @@ setMethod("evaluate", signature("Constraint", "TwoStageDesign"),
               evaluate(s@score, design, optimization, ...) - s@rhs
           })
 
-#' @rdname Constraints
-#' @export
-setMethod("show", signature(object = "Constraint"),
-          function(object) cat(class(object)[1]))
+setMethod("print", signature('UnconditionalConstraint'), function(x, ...) {
+    glue::glue("{print(x@score)} <= {x@rhs}")
+})
+
+setMethod("print", signature('ConditionalConstraint'), function(x, ...) {
+    glue::glue("{print(x@score)}(x1) <= {x@rhs} for x1 in [c1f,c1e]")
+})
+
+setMethod("show", signature(object = "Constraint"), function(object) {
+    cat(print(object), "\n")
+})
 
 
 
@@ -76,12 +83,12 @@ setMethod("<=", signature("ConditionalScore", "numeric"),
 #' @rdname Constraints
 #' @export
 setMethod(">=", signature("ConditionalScore", "numeric"),
-          function(e1, e2) new("ConditionalConstraint", score = composite({-1*e1}), rhs = -e2))
+          function(e1, e2) new("ConditionalConstraint", score = composite({-e1}), rhs = -e2))
 
 #' @rdname Constraints
 #' @export
 setMethod("<=", signature("numeric", "ConditionalScore"),
-          function(e1, e2) new("ConditionalConstraint", score = composite({-1*e2}), rhs = -e1))
+          function(e1, e2) new("ConditionalConstraint", score = composite({-e2}), rhs = -e1))
 
 #' @rdname Constraints
 #' @export
